local core = CalendarExtras

local module = core.NewModule("Status Online")

 --[[ -------- Locals ------- ]]--

local listIsDirty

local assumeMissingAreOnline -- If true, treat players not found in guild/friend list as being online

local sortersModule

local listFriends = {}
local listGuild = {}
--local listChannels = setmetatable({}, meta) -- Need to figure out how to do this easily...

 --[[ ----------------------- ]]--

local buttons = {}
local function DimButton(button)
		local buttonName = button:GetName()
		local name = _G[buttonName.."Name"]
		buttons[1], buttons[2], buttons[3] = _G[buttonName.."Class"], _G[buttonName.."Status"], sortersModule and _G[buttonName.."Role"] or nil
		name:SetTextColor(0.6, 0.6, 0.6, 0.65)
		local r, g, b
		for _, button in ipairs(buttons) do
			r, g, b = button:GetTextColor()
			button:SetTextColor(r, g, b, 0.5)
		end
end

local function IsOnline(name)
	return listFriends[name] or listGuild[name] or (UnitInRaid(name) and UnitIsConnected(name)) or (UnitInParty(name) and UnitIsConnected(name)) --[[ or listChannels[name] ]]
end

function module:FRIENDLIST_UPDATE()
	for i = 1, GetNumFriends() do
		local name, _, _, _, online = GetFriendInfo(i)
		local status
		if (assumeMissingAreOnline and not online) or (not assumeMissingAreOnline and online) then
			status = true
		end
		if not listIsDirty and listFriends[name] ~= status then
			listIsDirty = true
		end
		listFriends[name] = status
	end
	if listIsDirty then
		listIsDirty = nil
		core.UpdateOpenList() -- This is a bit heavy call I guess, but I don't want to recolorize all the entries myself
	end
end

function module:GUILD_ROSTER_UPDATE()
	for i = 1, GetNumGuildMembers(true) do
		local name, _, _, _, _, _, _, _, online = GetGuildRosterInfo(i)
		local status
		if (assumeMissingAreOnline and not online) or (not assumeMissingAreOnline and online) then
			status = true
		end
		if not listIsDirty and listGuild[name] ~= status then
			listIsDirty = true
		end
		listGuild[name] = status
	end
	if listIsDirty then
		listIsDirty = nil
		core.UpdateOpenList()
	end
end

-- Dim buttons for the offline people. This fires after the original Calendar***EventInviteListScrollFrame_Update which changes the colors for the buttons, so no need to "undim" them
local function UpdateInviteListButtons(_, frame)
	for _, button in ipairs(frame.buttons) do
		local inviteIndex = button.inviteIndex
		if inviteIndex then
			local name = CalendarEventGetInvite(inviteIndex)
			local found = IsOnline(name)
			if (assumeMissingAreOnline and found) or (not assumeMissingAreOnline and not found) then
				DimButton(button)
			end
		end
	end
end

function module.OnInitialize()
	if core.modules["Sorters"] then
		sortersModule = core.modules["Sorters"]
	end

	core.RegisterCallback(module, "Post_Update", UpdateInviteListButtons)

	-- Need to refresh things if people go online/offline, hopefully these catch all things properly.
	core.RegisterCallback(module, "CalendarShow", function()
		module:RegisterEvent("GUILD_ROSTER_UPDATE")
		module:RegisterEvent("FRIENDLIST_UPDATE")
		module:GUILD_ROSTER_UPDATE()
		module:FRIENDLIST_UPDATE()
	end)       
	core.RegisterCallback(module, "CalendarHide", function()
		module:UnregisterEvent("GUILD_ROSTER_UPDATE")
		module:UnregisterEvent("FRIENDLIST_UPDATE")
	end)
	module:RegisterEvent("GUILD_ROSTER_UPDATE")
	module:RegisterEvent("FRIENDLIST_UPDATE")
	module:GUILD_ROSTER_UPDATE()
	module:FRIENDLIST_UPDATE()
	GuildRoster()
end

--[[
	Some of the channel functions I found
local channel, header, collapsed, channelNumber, count, active, category = GetChannelDisplayInfo(id)
local name, owner, moderator, muted, active, enabled = GetChannelRosterInfo(id, rosterIndex)
SetSelectedDisplayChannel(id)
CHANNEL_ROSTER_UPDATE(channelId)
]]